import { useEffect, useRef, useState } from "react";

const createWorker = (workerScript) => {
  const blob = new Blob([workerScript], { type: "application/javascript" });
  const url = URL.createObjectURL(blob);
  return new Worker(url);
};

const timerWorkerScript = `
  let liveSeconds;
  let isRunning = false;
  self.onmessage = function(e) {
    if (typeof e.data === 'number') {
      liveSeconds = e.data;
    } else if (e.data === 'start' && !isRunning) {
      isRunning = true;
      if (typeof liveSeconds !== 'undefined') {
        self.interval = setInterval(() => {
          liveSeconds -= 1;
          self.postMessage({ type: 'timer', value: liveSeconds });
          if (liveSeconds <= 0) {
            self.postMessage({ type: 'timer', value: 0 });
            clearInterval(self.interval);
            isRunning = false;
          }
        }, 1000);
      }
    } else if (e.data === 'stop') {
      clearInterval(self.interval);
      isRunning = false;
    }
  };
`;

const useTextOrderTimer = () => {
  const timerWorkersRef = useRef({});
  const [timerStates, setTimerStates] = useState({});

  useEffect(() => {
    return () => {
      Object.values(timerWorkersRef.current).forEach((worker) =>
        worker.terminate()
      );
    };
  }, []);

  const startAllModalHandlerTimer = (timerId, duration, onTimeout) => {
    if (timerWorkersRef.current[timerId]) {
      timerWorkersRef.current[timerId].terminate();
    }

    const worker = createWorker(timerWorkerScript);
    worker.onmessage = (e) => {
      const { type, value } = e.data;
      if (type === "timer") {
        setTimerStates((prevStates) => ({
          ...prevStates,
          [timerId]: value,
        }));
        if (value === 0) {
          onTimeout();
          delete timerWorkersRef.current[timerId];
        }
      }
    };

    timerWorkersRef.current[timerId] = worker;
    setTimerStates((prevStates) => ({
      ...prevStates,
      [timerId]: duration,
    }));
    worker.postMessage(duration);
    worker.postMessage("start");
  };

  const stoptAllModalHandlerTimer = (timerId) => {
    if (timerWorkersRef.current[timerId]) {
      timerWorkersRef.current[timerId].postMessage("stop");
      timerWorkersRef.current[timerId].terminate();
      delete timerWorkersRef.current[timerId];
    }
    setTimerStates((prevStates) => {
      const newStates = { ...prevStates };
      delete newStates[timerId];
      return newStates;
    });
  };

  return { startAllModalHandlerTimer, stoptAllModalHandlerTimer, timerStates };
};

export default useTextOrderTimer;
